@@ -3,7 +3,6 @@ __all__ = [
 ]
 
 
-import os
 from dataclasses import InitVar, dataclass, field
 from pathlib import Path
 from typing import Any, Dict, Literal, MutableMapping, Optional, Tuple, Union, overload
@@ -13,7 +12,7 @@ from beet.core.utils import FileSystemPath, extra_field
 
 from .directive import Directive, RequireDirective, get_builtin_directives
 from .extract import MarkdownExtractor, TextExtractor
-from .serialize import MarkdownSerializer, TextSerializer
+from .serialize import ExternalFilesManager, MarkdownSerializer, TextSerializer
 
 
 @dataclass
@@ -105,7 +104,7 @@ class Document:
     def get_markdown(
         self,
         emit_external_files: Literal[True],
-        external_prefix: str = "",
+        prefix: str = "",
     ) -> Tuple[str, Dict[str, File[Any, Any]]]:
         ...
 
@@ -116,16 +115,18 @@ class Document:
     def get_markdown(
         self,
         emit_external_files: bool = False,
-        external_prefix: str = "",
+        prefix: str = "",
     ) -> Union[str, Tuple[str, Dict[str, File[Any, Any]]]]:
         """Turn the data pack and the resource pack into markdown."""
-        external_files = {} if emit_external_files else None
+        external_files: Optional[Dict[str, File[Any, Any]]] = (
+            {} if emit_external_files else None
+        )
 
         content = self.markdown_serializer.serialize(
             assets=self.assets,
             data=self.data,
             external_files=external_files,
-            external_prefix=external_prefix,
+            external_prefix=prefix,
         )
 
         if external_files is None:
@@ -143,21 +144,14 @@ class Document:
 
         if path.suffix == ".md":
             if external_files:
-                external_files = Path(external_files).resolve()
-
-                prefix = os.path.relpath(external_files, path.parent)
-                prefix = prefix.replace(os.path.sep, "/")
-                prefix = "" if prefix == "." else prefix.rstrip("/") + "/"
-
-                content, files = self.get_markdown(
-                    emit_external_files=True,
-                    external_prefix=prefix,
-                )
-
-                if files:
-                    external_files.mkdir(parents=True, exist_ok=True)
-                    for filename, file_instance in files.items():
-                        file_instance.dump(external_files, path.parent / filename)
+                with ExternalFilesManager(
+                    Path(external_files).resolve(), path
+                ) as manager:
+                    content, files = self.get_markdown(
+                        emit_external_files=True,
+                        prefix=manager.external_prefix,
+                    )
+                    manager.external_files.update(files)
             else:
                 content = self.get_markdown()
         else:
