@@ -7,9 +7,8 @@ __all__ = [
 import subprocess
 from typing import Iterable, List, Optional, cast
 
-from beet import Context
+from beet import Context, Plugin
 from beet.core.utils import JsonDict
-from beet.toolchain.context import Plugin
 
 from .document import Document
 
@@ -41,8 +40,13 @@ def lectern(
                 document.load(path)
 
         for arguments in scripts:
-            stdout = subprocess.check_output(arguments, cwd=ctx.directory).decode()
-            document.add_text(stdout)
+            result = subprocess.run(
+                arguments,
+                cwd=ctx.directory,
+                check=True,
+                stdout=subprocess.PIPE,
+            )
+            document.add_text(result.stdout.decode())
 
         yield
 
